@@ -120,6 +120,7 @@ struct NvKmsKapiDeviceResourcesInfo {
 
         NvU32 pitchAlignment;
 
+        NvU32 hasVideoMemory;
     } caps;
 
 };
@@ -483,10 +484,10 @@ struct NvKmsKapiFunctionsTable {
     );
 
     /*!
-     * Allocate some unformatted memory of the specified size.
+     * Allocate some unformatted video memory of the specified size.
      *
-     * This function allocates displayable (Usually vidmem) memory on the
-     * specified GPU. It should be suitable for mapping on the CPU as a pitch
+     * This function allocates video memory on the specified GPU.
+     * It should be suitable for mapping on the CPU as a pitch
      * linear surface.
      *
      * \param [in] device  A device allocated using allocateDevice().
@@ -495,7 +496,24 @@ struct NvKmsKapiFunctionsTable {
      *
      * \return An valid memory handle on success, NULL on failure.
      */
-    struct NvKmsKapiMemory* NVKMS_KAPI_CALL (*allocateMemory)
+    struct NvKmsKapiMemory* NVKMS_KAPI_CALL (*allocateVideoMemory)
+    (
+        struct NvKmsKapiDevice *device, NvU64 size
+    );
+
+    /*!
+     * Allocate some unformatted system memory of the specified size.
+     *
+     * This function allocates system memory . It should be suitable
+     * for mapping on the CPU as a pitch linear surface.
+     *
+     * \param [in] device  A device allocated using allocateDevice().
+     *
+     * \param [in] size    Size, in bytes, of the memory to allocate.
+     *
+     * \return An valid memory handle on success, NULL on failure.
+     */
+    struct NvKmsKapiMemory* NVKMS_KAPI_CALL (*allocateSystemMemory)
     (
         struct NvKmsKapiDevice *device, NvU64 size
     );
@@ -689,6 +707,9 @@ struct NvKmsKapiFunctionsTable {
      * \param [out]  valid      Returns TRUE in this param if mode-timings of
      *                          mode at index modeIndex are valid on display.
      *
+     * \param [out]  preferredMode  Returns TRUE if this mode is marked as
+     *                              "preferred" by the EDID.
+     *
      * \return Value >= 1 if more modes are available, 0 if no more modes are
      *         available, and Value < 0 on failure.
      */
@@ -696,7 +717,8 @@ struct NvKmsKapiFunctionsTable {
     (
         struct NvKmsKapiDevice *device,
         NvKmsKapiDisplay display, NvU32 modeIndex,
-        struct NvKmsKapiDisplayMode *mode, NvBool *valid
+        struct NvKmsKapiDisplayMode *mode, NvBool *valid,
+        NvBool *preferredMode
     );
 
     /*!
