@@ -85,36 +85,175 @@ extern "C" {
 
 // Index of the 'on' bit (assuming that there is only one).
 // Even if multiple bits are 'on', result is in range of 0-31.
-#define BIT_IDX_32(n)                   \
-   ((((n) & 0xFFFF0000U)? 0x10: 0) |    \
-    (((n) & 0xFF00FF00U)? 0x08: 0) |    \
-    (((n) & 0xF0F0F0F0U)? 0x04: 0) |    \
-    (((n) & 0xCCCCCCCCU)? 0x02: 0) |    \
-    (((n) & 0xAAAAAAAAU)? 0x01: 0) )
+#define BIT_IDX_32(n)                            \
+   (((((n) & 0xFFFF0000U) != 0U) ? 0x10U: 0U) |  \
+    ((((n) & 0xFF00FF00U) != 0U) ? 0x08U: 0U) |  \
+    ((((n) & 0xF0F0F0F0U) != 0U) ? 0x04U: 0U) |  \
+    ((((n) & 0xCCCCCCCCU) != 0U) ? 0x02U: 0U) |  \
+    ((((n) & 0xAAAAAAAAU) != 0U) ? 0x01U: 0U) )
 
 // Index of the 'on' bit (assuming that there is only one).
 // Even if multiple bits are 'on', result is in range of 0-63.
-#define BIT_IDX_64(n)                              \
-   ((((n) & 0xFFFFFFFF00000000ULL)? 0x20: 0) |     \
-    (((n) & 0xFFFF0000FFFF0000ULL)? 0x10: 0) |     \
-    (((n) & 0xFF00FF00FF00FF00ULL)? 0x08: 0) |     \
-    (((n) & 0xF0F0F0F0F0F0F0F0ULL)? 0x04: 0) |     \
-    (((n) & 0xCCCCCCCCCCCCCCCCULL)? 0x02: 0) |     \
-    (((n) & 0xAAAAAAAAAAAAAAAAULL)? 0x01: 0) )
+#define BIT_IDX_64(n)                                       \
+   (((((n) & 0xFFFFFFFF00000000ULL) != 0U) ? 0x20U: 0U) |   \
+    ((((n) & 0xFFFF0000FFFF0000ULL) != 0U) ? 0x10U: 0U) |   \
+    ((((n) & 0xFF00FF00FF00FF00ULL) != 0U) ? 0x08U: 0U) |   \
+    ((((n) & 0xF0F0F0F0F0F0F0F0ULL) != 0U) ? 0x04U: 0U) |   \
+    ((((n) & 0xCCCCCCCCCCCCCCCCULL) != 0U) ? 0x02U: 0U) |   \
+    ((((n) & 0xAAAAAAAAAAAAAAAAULL) != 0U) ? 0x01U: 0U) )
 
 // Desctructive, assumes only one bit set in n32
 // Deprecated in favor of BIT_IDX_32.
 #define IDX_32(n32)                     \
 {                                       \
     NvU32 idx = 0;                      \
-    if ((n32) & 0xFFFF0000U) idx += 16; \
-    if ((n32) & 0xFF00FF00U) idx += 8;  \
-    if ((n32) & 0xF0F0F0F0U) idx += 4;  \
-    if ((n32) & 0xCCCCCCCCU) idx += 2;  \
-    if ((n32) & 0xAAAAAAAAU) idx += 1;  \
+    if (((n32) & 0xFFFF0000U) != 0U)    \
+    {                                   \
+         idx += 16U;                    \
+    }                                   \
+    if (((n32) & 0xFF00FF00U) != 0U)    \
+    {                                   \
+         idx += 8U;                     \
+    }                                   \
+    if (((n32) & 0xF0F0F0F0U) != 0U)    \
+    {                                   \
+         idx += 4U;                     \
+    }                                   \
+    if (((n32) & 0xCCCCCCCCU) != 0U)    \
+    {                                   \
+         idx += 2U;                     \
+    }                                   \
+    if (((n32) & 0xAAAAAAAAU) != 0U)    \
+    {                                   \
+         idx += 1U;                     \
+    }                                   \
     (n32) = idx;                        \
 }
 
+/*!
+ * DRF MACRO README:
+ *
+ * Glossary:
+ *      DRF: Device, Register, Field
+ *      FLD: Field
+ *      REF: Reference
+ *
+ * #define NV_DEVICE_OMEGA_REGISTER_ALPHA                   0xDEADBEEF
+ * #define NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_GAMMA             27:0
+ * #define NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA             31:28
+ * #define NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA_ZERO   0x00000000
+ * #define NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA_ONE    0x00000001
+ * #define NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA_TWO    0x00000002
+ * #define NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA_THREE  0x00000003
+ * #define NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA_FOUR   0x00000004
+ * #define NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA_FIVE   0x00000005
+ * #define NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA_SIX    0x00000006
+ * #define NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA_SEVEN  0x00000007
+ *
+ *
+ * Device = _DEVICE_OMEGA
+ *   This is the common "base" that a group of registers in a manual share
+ *
+ * Register = _REGISTER_ALPHA
+ *   Register for a given block of defines is the common root for one or more fields and constants
+ *
+ * Field(s) = _FIELD_GAMMA, _FIELD_ZETA
+ *   These are the bit ranges for a given field within the register
+ *   Fields are not required to have defined constant values (enumerations)
+ *
+ * Constant(s) = _ZERO, _ONE, _TWO, ...
+ *   These are named values (enums) a field can contain; the width of the constants should not be larger than the field width
+ *
+ * MACROS:
+ *
+ * DRF_SHIFT:
+ *      Bit index of the lower bound of a field
+ *      DRF_SHIFT(NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA) == 28
+ *
+ * DRF_SHIFT_RT:
+ *      Bit index of the higher bound of a field
+ *      DRF_SHIFT_RT(NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA) == 31
+ *
+ * DRF_MASK:
+ *      Produces a mask of 1-s equal to the width of a field
+ *      DRF_MASK(NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA) == 0xF (four 1s starting at bit 0)
+ *
+ * DRF_SHIFTMASK:
+ *      Produces a mask of 1s equal to the width of a field at the location of the field
+ *      DRF_SHIFTMASK(NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA) == 0xF0000000
+ *
+ * DRF_DEF:
+ *      Shifts a field constant's value to the correct field offset
+ *      DRF_DEF(_DEVICE_OMEGA, _REGISTER_ALPHA, _FIELD_ZETA, _THREE) == 0x30000000
+ *
+ * DRF_NUM:
+ *      Shifts a number to the location of a particular field
+ *      DRF_NUM(_DEVICE_OMEGA, _REGISTER_ALPHA, _FIELD_ZETA, 3) == 0x30000000
+ *      NOTE: If the value passed in is wider than the field, the value's high bits will be truncated
+ *
+ * DRF_SIZE:
+ *      Provides the width of the field in bits
+ *      DRF_SIZE(NV_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA) == 4
+ *
+ * DRF_VAL:
+ *      Provides the value of an input within the field specified
+ *      DRF_VAL(_DEVICE_OMEGA, _REGISTER_ALPHA, _FIELD_ZETA, 0xABCD1234) == 0xA
+ *      This is sort of like the inverse of DRF_NUM
+ *
+ * DRF_IDX...:
+ *      These macros are similar to the above but for fields that accept an index argumment
+ *
+ * FLD_SET_DRF:
+ *      Set the field bits in a given value with the given field constant
+ *      NvU32 x = 0x00001234;
+ *      x = FLD_SET_DRF(_DEVICE_OMEGA, _REGISTER_ALPHA, _FIELD_ZETA, _THREE, x);
+ *      x == 0x30001234;
+ *
+ * FLD_SET_DRF_NUM:
+ *      Same as FLD_SET_DRF but instead of using a field constant a literal/variable is passed in
+ *      NvU32 x = 0x00001234;
+ *      x = FLD_SET_DRF_NUM(_DEVICE_OMEGA, _REGISTER_ALPHA, _FIELD_ZETA, 0xF, x);
+ *      x == 0xF0001234;
+ *
+ * FLD_IDX...:
+ *      These macros are similar to the above but for fields that accept an index argumment
+ *
+ * FLD_TEST_DRF:
+ *      Test if location specified by drf in 'v' has the same value as NV_drfc
+ *      FLD_TEST_DRF(_DEVICE_OMEGA, _REGISTER_ALPHA, _FIELD_ZETA, _THREE, 0x3000ABCD) == NV_TRUE
+ *
+ * FLD_TEST_DRF_NUM:
+ *      Test if locations specified by drf in 'v' have the same value as n
+ *      FLD_TEST_DRF_NUM(_DEVICE_OMEGA, _REGISTER_ALPHA, _FIELD_ZETA, 0x3, 0x3000ABCD) == NV_TRUE
+ *
+ * REF_DEF:
+ *      Like DRF_DEF but maintains full symbol name (use in cases where "NV" is not prefixed to the field)
+ *      REF_DEF(SOME_OTHER_PREFIX_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA, _THREE) == 0x30000000
+ *
+ * REF_VAL:
+ *      Like DRF_VAL but maintains full symbol name (use in cases where "NV" is not prefixed to the field)
+ *      REF_VAL(SOME_OTHER_PREFIX_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA, 0xABCD1234) == 0xA
+ *
+ * REF_NUM:
+ *      Like DRF_NUM but maintains full symbol name (use in cases where "NV" is not prefixed to the field)
+ *      REF_NUM(SOME_OTHER_PREFIX_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA, 0xA) == 0xA00000000
+ *
+ * FLD_SET_REF_NUM:
+ *      Like FLD_SET_DRF_NUM but maintains full symbol name (use in cases where "NV" is not prefixed to the field)
+ *      NvU32 x = 0x00001234;
+ *      x = FLD_SET_REF_NUM(SOME_OTHER_PREFIX_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA, 0xF, x);
+ *      x == 0xF0001234;
+ *
+ * FLD_TEST_REF:
+ *      Like FLD_TEST_DRF but maintains full symbol name (use in cases where "NV" is not prefixed to the field)
+ *      FLD_TEST_REF(SOME_OTHER_PREFIX_DEVICE_OMEGA_REGISTER_ALPHA_FIELD_ZETA, _THREE, 0x3000ABCD) == NV_TRUE
+ *
+ * Other macros:
+ *      There a plethora of other macros below that extend the above (notably Multi-Word (MW), 64-bit, and some
+ *      reg read/write variations). I hope these are self explanatory. If you have a need to use them, you
+ *      probably have some knowlege of how they work.
+ */
+
 // tegra mobile uses nvmisc_macros.h and can't access nvmisc.h... and sometimes both get included.
 #ifndef _NVMISC_MACROS_H
 // Use Coverity Annotation to mark issues as false positives/ignore when using single bit defines.
@@ -574,9 +713,9 @@ nvPrevPow2_U32(const NvU32 x )
 #define FOR_EACH_INDEX_IN_MASK(maskWidth,index,mask)        \
 {                                                           \
     NvU##maskWidth lclMsk = (NvU##maskWidth)(mask);         \
-    for (index = 0; lclMsk != 0; index++, lclMsk >>= 1)     \
+    for ((index) = 0U; lclMsk != 0U; (index)++, lclMsk >>= 1U)\
     {                                                       \
-        if (((NvU##maskWidth)NVBIT64(0) & lclMsk) == 0)     \
+        if (((NvU##maskWidth)NVBIT64(0) & lclMsk) == 0U)    \
         {                                                   \
             continue;                                       \
         }
@@ -637,7 +776,7 @@ nvPrevPow2_U32(const NvU32 x )
 // Performs a rounded division of b into a (unsigned). For SIGNED version of
 // NV_ROUNDED_DIV() macro check the comments in http://nvbugs/769777.
 //
-#define NV_UNSIGNED_ROUNDED_DIV(a,b)    (((a) + ((b) / 2)) / (b))
+#define NV_UNSIGNED_ROUNDED_DIV(a,b)    (((a) + ((b) / 2U)) / (b))
 
 /*!
  * Performs a ceiling division of b into a (unsigned).  A "ceiling" division is
