@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015 NVIDIA Corporation
+    Copyright (c) 2015-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -24,7 +24,6 @@
 #include "uvm8_api.h"
 #include "uvm8_global.h"
 #include "uvm8_gpu_replayable_faults.h"
-#include "uvm8_init.h"
 #include "uvm8_tools_init.h"
 #include "uvm8_lock.h"
 #include "uvm8_test.h"
@@ -37,6 +36,9 @@
 #include "uvm8_hmm.h"
 #include "uvm8_mem.h"
 
+#define NVIDIA_UVM_DEVICE_NAME          "nvidia-uvm"
+
+static dev_t g_uvm_base_dev;
 static struct cdev g_uvm_cdev;
 
 // List of fault service contexts for CPU faults
@@ -122,6 +124,11 @@ static int uvm_open(struct inode *inode, struct file *filp)
     return -nv_status_to_errno(status);
 }
 
+static int uvm_open_entry(struct inode *inode, struct file *filp)
+{
+   UVM_ENTRY_RET(uvm_open(inode, filp));
+}
+
 static void uvm_release_deferred(void *data)
 {
     uvm_va_space_t *va_space = data;
@@ -170,6 +177,11 @@ static int uvm_release(struct inode *inode, struct file *filp)
     return 0;
 }
 
+static int uvm_release_entry(struct inode *inode, struct file *filp)
+{
+   UVM_ENTRY_RET(uvm_release(inode, filp));
+}
+
 static void uvm_destroy_vma_managed(struct vm_area_struct *vma, bool is_uvm_teardown)
 {
     uvm_va_range_t *va_range, *va_range_next;
@@ -220,6 +232,11 @@ static vm_fault_t uvm_vm_fault_sigbus(struct vm_area_struct *vma, struct vm_faul
     return VM_FAULT_SIGBUS;
 }
 
+static vm_fault_t uvm_vm_fault_sigbus_entry(struct vm_area_struct *vma, struct vm_fault *vmf)
+{
+    UVM_ENTRY_RET(uvm_vm_fault_sigbus(vma, vmf));
+}
+
 static vm_fault_t uvm_vm_fault_sigbus_wrapper(struct vm_fault *vmf)
 {
 #if defined(NV_VM_OPS_FAULT_REMOVED_VMA_ARG)
@@ -229,12 +246,17 @@ static vm_fault_t uvm_vm_fault_sigbus_wrapper(struct vm_fault *vmf)
 #endif
 }
 
+static vm_fault_t uvm_vm_fault_sigbus_wrapper_entry(struct vm_fault *vmf)
+{
+    UVM_ENTRY_RET(uvm_vm_fault_sigbus_wrapper(vmf));
+}
+
 static struct vm_operations_struct uvm_vm_ops_disabled =
 {
 #if defined(NV_VM_OPS_FAULT_REMOVED_VMA_ARG)
-    .fault = uvm_vm_fault_sigbus_wrapper
+    .fault = uvm_vm_fault_sigbus_wrapper_entry
 #else
-    .fault = uvm_vm_fault_sigbus
+    .fault = uvm_vm_fault_sigbus_entry
 #endif
 };
 
@@ -398,6 +420,11 @@ out:
     uvm_record_unlock_mmap_sem_write(&current->mm->mmap_sem);
 }
 
+static void uvm_vm_open_managed_entry(struct vm_area_struct *vma)
+{
+   UVM_ENTRY_VOID(uvm_vm_open_managed(vma));
+}
+
 static void uvm_vm_close_managed(struct vm_area_struct *vma)
 {
     uvm_va_space_t *va_space = uvm_va_space_get(vma->vm_file);
@@ -426,7 +453,7 @@ static void uvm_vm_close_managed(struct vm_area_struct *vma)
 
     // Notify GPU address spaces that the fault buffer needs to be flushed to avoid finding stale entries
     // that can be attributed to new VA ranges reallocated at the same address
-    for_each_gpu_in_mask(gpu, &va_space->registered_gpu_va_spaces) {
+    for_each_va_space_gpu_in_mask(gpu, va_space, &va_space->registered_gpu_va_spaces) {
         uvm_gpu_va_space_t *gpu_va_space = uvm_gpu_va_space_get(va_space, gpu);
         UVM_ASSERT(gpu_va_space);
 
@@ -438,6 +465,11 @@ static void uvm_vm_close_managed(struct vm_area_struct *vma)
         uvm_record_unlock_mmap_sem_write(&current->mm->mmap_sem);
 }
 
+static void uvm_vm_close_managed_entry(struct vm_area_struct *vma)
+{
+    UVM_ENTRY_VOID(uvm_vm_close_managed(vma));
+}
+
 static vm_fault_t uvm_vm_fault(struct vm_area_struct *vma, struct vm_fault *vmf)
 {
     uvm_va_space_t *va_space = uvm_va_space_get(vma->vm_file);
@@ -448,11 +480,21 @@ static vm_fault_t uvm_vm_fault(struct vm_area_struct *vma, struct vm_fault *vmf)
     bool tools_enabled;
     bool major_fault = false;
     uvm_service_block_context_t *service_context;
+    uvm_global_processor_mask_t gpus_to_check_for_ecc;
 
     if (status != NV_OK)
         goto convert_error;
 
-    if (!uvm_down_read_trylock(&g_uvm_global.pm.lock)) {
+    // TODO: Bug 2583279: Lock tracking is disabled for the power management
+    // lock in order to suppress reporting of a lock policy violation.
+    // The violation consists in acquiring the power management lock multiple
+    // times, and it is manifested as an error during release. The
+    // re-acquisition of the power management locks happens upon re-entry in the
+    // UVM module, and it is benign on itself, but when combined with certain
+    // power management scenarios, it is indicative of a potential deadlock.
+    // Tracking will be re-enabled once the power management locking strategy is
+    // modified to avoid deadlocks.
+    if (!uvm_down_read_trylock_no_tracking(&g_uvm_global.pm.lock)) {
         status = NV_ERR_BUSY_RETRY;
         goto convert_error;
     }
@@ -478,7 +520,7 @@ static vm_fault_t uvm_vm_fault(struct vm_area_struct *vma, struct vm_fault *vmf)
                 do_sleep = true;
 
             if (do_sleep)
-                uvm_tools_record_throttling_start(va_space, fault_addr, UVM_CPU_ID);
+                uvm_tools_record_throttling_start(va_space, fault_addr, UVM_ID_CPU);
 
             // Drop the VA space lock while we sleep
             uvm_va_space_up_read(va_space);
@@ -497,12 +539,12 @@ static vm_fault_t uvm_vm_fault(struct vm_area_struct *vma, struct vm_fault *vmf)
         uvm_va_space_down_read(va_space);
 
         if (do_sleep)
-            uvm_tools_record_throttling_end(va_space, fault_addr, UVM_CPU_ID);
+            uvm_tools_record_throttling_end(va_space, fault_addr, UVM_ID_CPU);
 
         status = uvm_va_block_find_create(va_space, fault_addr, &va_block);
         if (status != NV_OK) {
             UVM_ASSERT_MSG(status == NV_ERR_NO_MEMORY, "status: %s\n", nvstatusToString(status));
-            goto out;
+            break;
         }
 
         // Watch out, current->mm might not be vma->vm_mm
@@ -512,7 +554,6 @@ static vm_fault_t uvm_vm_fault(struct vm_area_struct *vma, struct vm_fault *vmf)
         status = uvm_va_block_cpu_fault(va_block, fault_addr, is_write, service_context);
     } while (status == NV_WARN_MORE_PROCESSING_REQUIRED);
 
-out:
     if (status != NV_OK) {
         UvmEventFatalReason reason;
 
@@ -524,20 +565,19 @@ out:
 
     tools_enabled = va_space->tools.enabled;
 
-    if (status == NV_OK)
-        uvm_gpu_retain_mask(&service_context->cpu_fault.gpus_to_check_for_ecc);
+    if (status == NV_OK) {
+        uvm_va_space_global_gpus_in_mask(va_space,
+                                         &gpus_to_check_for_ecc,
+                                         &service_context->cpu_fault.gpus_to_check_for_ecc);
+        uvm_global_mask_retain(&gpus_to_check_for_ecc);
+    }
 
     uvm_va_space_up_read(va_space);
     uvm_record_unlock_mmap_sem_read(&vma->vm_mm->mmap_sem);
 
     if (status == NV_OK) {
-        uvm_gpu_t *gpu;
-        for_each_gpu_in_mask(gpu, &service_context->cpu_fault.gpus_to_check_for_ecc) {
-            status = uvm_gpu_check_ecc_error(gpu);
-            if (status != NV_OK)
-                break;
-        }
-        uvm_gpu_release_mask(&service_context->cpu_fault.gpus_to_check_for_ecc);
+        status = uvm_global_mask_check_ecc_error(&gpus_to_check_for_ecc);
+        uvm_global_mask_release(&gpus_to_check_for_ecc);
     }
 
     if (tools_enabled)
@@ -550,7 +590,8 @@ out:
     put_cpu_fault_service_context(service_context);
 
 unlock:
-    uvm_up_read(&g_uvm_global.pm.lock);
+    // TODO: Bug 2583279: See the comment above the matching lock acquisition
+    uvm_up_read_no_tracking(&g_uvm_global.pm.lock);
 
 convert_error:
     switch (status) {
@@ -564,6 +605,12 @@ convert_error:
     }
 }
 
+
+static vm_fault_t uvm_vm_fault_entry(struct vm_area_struct *vma, struct vm_fault *vmf)
+{
+    UVM_ENTRY_RET(uvm_vm_fault(vma, vmf));
+}
+
 static vm_fault_t uvm_vm_fault_wrapper(struct vm_fault *vmf)
 {
 #if defined(NV_VM_OPS_FAULT_REMOVED_VMA_ARG)
@@ -573,17 +620,22 @@ static vm_fault_t uvm_vm_fault_wrapper(struct vm_fault *vmf)
 #endif
 }
 
+static vm_fault_t uvm_vm_fault_wrapper_entry(struct vm_fault *vmf)
+{
+    UVM_ENTRY_RET(uvm_vm_fault_wrapper(vmf));
+}
+
 static struct vm_operations_struct uvm_vm_ops_managed =
 {
-    .open         = uvm_vm_open_managed,
-    .close        = uvm_vm_close_managed,
+    .open         = uvm_vm_open_managed_entry,
+    .close        = uvm_vm_close_managed_entry,
 
 #if defined(NV_VM_OPS_FAULT_REMOVED_VMA_ARG)
-    .fault        = uvm_vm_fault_wrapper,
-    .page_mkwrite = uvm_vm_fault_wrapper,
+    .fault        = uvm_vm_fault_wrapper_entry,
+    .page_mkwrite = uvm_vm_fault_wrapper_entry,
 #else
-    .fault        = uvm_vm_fault,
-    .page_mkwrite = uvm_vm_fault,
+    .fault        = uvm_vm_fault_entry,
+    .page_mkwrite = uvm_vm_fault_entry,
 #endif
 };
 
@@ -617,8 +669,10 @@ static void uvm_vm_open_semaphore_pool(struct vm_area_struct *vma)
     if (is_fork) {
         // If we forked, leave the parent vma alone.
         uvm_disable_vma(vma);
+
         // uvm_disable_vma unmaps in the parent as well; remap it.
-        uvm_processor_mask_clear(&va_range->semaphore_pool.mem->mapped_on, UVM_CPU_ID);
+        uvm_global_processor_mask_clear(&va_range->semaphore_pool.mem->mapped_on, UVM_GLOBAL_ID_CPU);
+
         status = uvm_mem_map_cpu(va_range->semaphore_pool.mem, origin_vma);
         if (status != NV_OK) {
             UVM_DBG_PRINT("Failed to remap semaphore pool to CPU for parent after fork; status = %d (%s)",
@@ -638,6 +692,11 @@ static void uvm_vm_open_semaphore_pool(struct vm_area_struct *vma)
     uvm_record_unlock_mmap_sem_write(&current->mm->mmap_sem);
 }
 
+static void uvm_vm_open_semaphore_pool_entry(struct vm_area_struct *vma)
+{
+   UVM_ENTRY_VOID(uvm_vm_open_semaphore_pool(vma));
+}
+
 // vm operations on semaphore pool allocations only control CPU mappings. Unmapping GPUs,
 // freeing the allocation, and destroying the va_range are handled by UVM_FREE.
 static void uvm_vm_close_semaphore_pool(struct vm_area_struct *vma)
@@ -657,15 +716,20 @@ static void uvm_vm_close_semaphore_pool(struct vm_area_struct *vma)
         uvm_record_unlock_mmap_sem_write(&current->mm->mmap_sem);
 }
 
+static void uvm_vm_close_semaphore_pool_entry(struct vm_area_struct *vma)
+{
+   UVM_ENTRY_VOID(uvm_vm_close_semaphore_pool(vma));
+}
+
 static struct vm_operations_struct uvm_vm_ops_semaphore_pool =
 {
-    .open         = uvm_vm_open_semaphore_pool,
-    .close        = uvm_vm_close_semaphore_pool,
+    .open         = uvm_vm_open_semaphore_pool_entry,
+    .close        = uvm_vm_close_semaphore_pool_entry,
 
 #if defined(NV_VM_OPS_FAULT_REMOVED_VMA_ARG)
-    .fault        = uvm_vm_fault_sigbus_wrapper,
+    .fault        = uvm_vm_fault_sigbus_wrapper_entry,
 #else
-    .fault        = uvm_vm_fault_sigbus,
+    .fault        = uvm_vm_fault_sigbus_entry,
 #endif
 };
 
@@ -781,6 +845,41 @@ out:
     return ret;
 }
 
+static int uvm_mmap_entry(struct file *filp, struct vm_area_struct *vma)
+{
+   UVM_ENTRY_RET(uvm_mmap(filp, vma));
+}
+
+static NV_STATUS uvm_api_initialize(UVM_INITIALIZE_PARAMS *params, struct file *filp)
+{
+    NV_STATUS status = NV_OK;
+    uvm_va_space_t *va_space = uvm_va_space_get(filp);
+
+    if ((params->flags & ~UVM_INIT_FLAGS_MASK))
+        return NV_ERR_INVALID_ARGUMENT;
+
+    uvm_down_write_mmap_sem(&current->mm->mmap_sem);
+    uvm_va_space_down_write(va_space);
+
+    if (va_space->initialized) {
+        // Already initialized - check if parameters match
+        if (params->flags != va_space->initialization_flags)
+            status = NV_ERR_INVALID_ARGUMENT;
+    }
+    else {
+        va_space->initialization_flags = params->flags;
+
+        status = uvm_hmm_mirror_register(va_space);
+        if (status == NV_OK)
+            va_space->initialized = true;
+    }
+
+    uvm_va_space_up_write(va_space);
+    uvm_up_write_mmap_sem(&current->mm->mmap_sem);
+
+    return status;
+}
+
 static long uvm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 {
     switch (cmd)
@@ -789,7 +888,6 @@ static long uvm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
             return 0;
 
         UVM_ROUTE_CMD_STACK(UVM_INITIALIZE,                     uvm_api_initialize);
-        UVM_ROUTE_CMD_STACK(UVM_IS_8_SUPPORTED,                 uvm_api_is_8_supported);
         UVM_ROUTE_CMD_STACK(UVM_PAGEABLE_MEM_ACCESS,            uvm_api_pageable_mem_access);
         UVM_ROUTE_CMD_STACK(UVM_PAGEABLE_MEM_ACCESS_ON_GPU,     uvm_api_pageable_mem_access_on_gpu);
         UVM_ROUTE_CMD_STACK(UVM_REGISTER_GPU,                   uvm_api_register_gpu);
@@ -849,30 +947,52 @@ static long uvm_unlocked_ioctl(struct file *filp, unsigned int cmd, unsigned lon
     return ret;
 }
 
+static long uvm_unlocked_ioctl_entry(struct file *filp, unsigned int cmd, unsigned long arg)
+{
+   UVM_ENTRY_RET(uvm_unlocked_ioctl(filp, cmd, arg));
+}
+
 static const struct file_operations uvm_fops =
 {
-    .open            = uvm_open,
-    .release         = uvm_release,
-    .mmap            = uvm_mmap,
-    .unlocked_ioctl  = uvm_unlocked_ioctl,
-#if NVCPU_IS_X86_64 && defined(NV_FILE_OPERATIONS_HAS_COMPAT_IOCTL)
-    .compat_ioctl    = uvm_unlocked_ioctl,
+    .open            = uvm_open_entry,
+    .release         = uvm_release_entry,
+    .mmap            = uvm_mmap_entry,
+    .unlocked_ioctl  = uvm_unlocked_ioctl_entry,
+#if NVCPU_IS_X86_64
+    .compat_ioctl    = uvm_unlocked_ioctl_entry,
 #endif
     .owner           = THIS_MODULE,
 };
 
-int uvm8_init(dev_t uvm_base_dev)
+bool uvm_file_is_nvidia_uvm(struct file *filp)
 {
+    return (filp != NULL) && (filp->f_op == &uvm_fops);
+}
+
+static int uvm_init(void)
+{
+    bool allocated_dev = false;
     bool initialized_globals = false;
     bool added_device = false;
     bool initialized_tools = false;
-    int ret = -ENODEV;
-    dev_t uvm_dev = MKDEV(MAJOR(uvm_base_dev), NVIDIA_UVM_PRIMARY_MINOR_NUMBER);
     NV_STATUS status;
+    dev_t uvm_dev;
+
+    int ret = alloc_chrdev_region(&g_uvm_base_dev,
+                                  0,
+                                  NVIDIA_UVM_NUM_MINOR_DEVICES,
+                                  NVIDIA_UVM_DEVICE_NAME);
+    if (ret != 0) {
+        UVM_ERR_PRINT("alloc_chrdev_region failed: %d\n", ret);
+        goto error;
+    }
+    uvm_dev = MKDEV(MAJOR(g_uvm_base_dev), NVIDIA_UVM_PRIMARY_MINOR_NUMBER);
+    allocated_dev = true;
 
     status = uvm_global_init();
     if (status != NV_OK) {
         UVM_ERR_PRINT("uvm_global_init() failed: %s\n", nvstatusToString(status));
+        ret = -ENODEV;
         goto error;
     }
     initialized_globals = true;
@@ -885,7 +1005,7 @@ int uvm8_init(dev_t uvm_base_dev)
     }
     added_device = true;
 
-    ret = uvm_tools_init(uvm_base_dev);
+    ret = uvm_tools_init(g_uvm_base_dev);
     if (ret != 0) {
         UVM_ERR_PRINT("uvm_tools_init() failed: %d\n", ret);
         goto error;
@@ -900,6 +1020,11 @@ int uvm8_init(dev_t uvm_base_dev)
 
     uvm_hmm_init();
 
+    pr_info("Loaded the UVM driver, major device number %d.\n", MAJOR(g_uvm_base_dev));
+
+    if (uvm_enable_builtin_tests)
+        pr_info("Built-in UVM tests are enabled. This is a security risk.\n");
+
     return 0;
 
 error:
@@ -914,49 +1039,40 @@ error:
     if (initialized_globals)
         uvm_global_exit();
 
+    if (allocated_dev)
+        unregister_chrdev_region(g_uvm_base_dev, NVIDIA_UVM_NUM_MINOR_DEVICES);
+
+    UVM_ERR_PRINT("uvm init failed: %d\n", ret);
+
     return ret;
 }
 
-void uvm8_exit(void)
+static int __init uvm_init_entry(void)
+{
+   UVM_ENTRY_RET(uvm_init());
+}
+
+static void uvm_exit(void)
 {
     free_cpu_service_block_context_list();
     uvm_tools_exit();
     cdev_del(&g_uvm_cdev);
 
     uvm_global_exit();
+
+    unregister_chrdev_region(g_uvm_base_dev, NVIDIA_UVM_NUM_MINOR_DEVICES);
+
+    pr_info("Unloaded the UVM driver.\n");
 }
 
-NV_STATUS uvm8_initialize(UVM_INITIALIZE_PARAMS *params, struct file *filp)
+static void __exit uvm_exit_entry(void)
 {
-    NV_STATUS status = NV_OK;
-    uvm_va_space_t *va_space = uvm_va_space_get(filp);
-
-    if ((params->flags & ~UVM_INIT_FLAGS_MASK))
-        return NV_ERR_INVALID_ARGUMENT;
-
-    uvm_down_write_mmap_sem(&current->mm->mmap_sem);
-    uvm_va_space_down_write(va_space);
-
-    if (va_space->initialized) {
-        // Already initialized - check if parameters match
-        if (params->flags != va_space->initialization_flags)
-            status = NV_ERR_INVALID_ARGUMENT;
-    }
-    else {
-        va_space->initialization_flags = params->flags;
-
-        status = uvm_hmm_mirror_register(va_space);
-        if (status == NV_OK)
-            va_space->initialized = true;
-    }
-
-    uvm_va_space_up_write(va_space);
-    uvm_up_write_mmap_sem(&current->mm->mmap_sem);
-
-    return status;
+   UVM_ENTRY_VOID(uvm_exit());
 }
 
-bool uvm_file_is_nvidia_uvm(struct file *filp)
-{
-    return (filp != NULL) && (filp->f_op == &uvm_fops);
-}
+module_init(uvm_init_entry);
+module_exit(uvm_exit_entry);
+
+MODULE_LICENSE("Dual MIT/GPL");
+MODULE_INFO(supported, "external");
+
