@@ -154,7 +154,7 @@ struct uvm_service_block_context_struct
         // We need one mask per processor because we may coalesce faults that
         // trigger migrations to different processors.
         uvm_page_mask_t new_residency;
-    } per_processor_masks[UVM_MAX_PROCESSORS];
+    } per_processor_masks[UVM_ID_MAX_PROCESSORS];
 
     // State used by the VA block routines called by the servicing routine
     uvm_va_block_context_t block_context;
@@ -540,9 +540,16 @@ struct uvm_gpu_struct
     // user can create a lot of va spaces and register the gpu with them).
     atomic64_t retained_count;
 
-    // A unique uvm gpu id in range [1, UVM_MAX_PROCESSORS)
+    // GPU information and provided by RM (architecture, implementation,
+    // hardware classes, etc.)
+    UvmGpuInfo rm_info;
+
+    // A unique uvm gpu id in range [1, UVM_ID_MAX_PROCESSORS)
     uvm_gpu_id_t id;
 
+    // A unique uvm global_gpu id in range [1, UVM_GLOBAL_ID_MAX_PROCESSORS)
+    uvm_global_gpu_id_t global_id;
+
     // The gpu's uuid
     NvProcessorUuid uuid;
 
@@ -580,24 +587,6 @@ struct uvm_gpu_struct
     // Should be UVM_GPU_MAGIC_VALUE. Used for memory checking.
     NvU64 magic;
 
-    // Gpu architecture; NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_*
-    NvU32 architecture;
-
-    // Gpu implementation; NV2080_CTRL_MC_ARCH_INFO_IMPLEMENTATION_*
-    NvU32 implementation;
-
-    // Host (gpfifo) class; *_CHANNEL_GPFIFO_*, e.g. KEPLER_CHANNEL_GPFIFO_A
-    NvU32 host_class;
-
-    // Copy engine (dma) class; *_DMA_COPY_*, e.g. KEPLER_DMA_COPY_A
-    NvU32 ce_class;
-
-    // Fault buffer class; *_FAULT_BUFFER_*, e.g. MAXWELL_FAULT_BUFFER_A
-    NvU32 fault_buffer_class;
-
-    // Access counter buffer class; ACCESS_COUNTER_NOTIFY_BUFFER
-    NvU32 access_counter_buffer_class;
-
     // Hardware Abstraction Layer
     uvm_host_hal_t *host_hal;
     uvm_ce_hal_t *ce_hal;
@@ -605,12 +594,16 @@ struct uvm_gpu_struct
     uvm_fault_buffer_hal_t *fault_buffer_hal;
     uvm_access_counter_buffer_hal_t *access_counter_buffer_hal;
 
-    // The amount of video memory the GPU has in total, in bytes.
-    NvU64 vidmem_size;
+    struct
+    {
+        // The amount of memory the GPU has in total, in bytes. If the GPU is in
+        // ZeroFB testing mode, this will be 0.
+        NvU64 size;
 
-    // Max (inclusive) physical address of this GPU's vidmem that the driver can
-    // allocate through PMM (PMA).
-    NvU64 vidmem_max_allocatable_address;
+        // Max (inclusive) physical address of this GPU's memory that the driver
+        // can allocate through PMM (PMA).
+        NvU64 max_allocatable_address;
+    } mem_info;
 
     struct
     {
@@ -661,15 +654,20 @@ struct uvm_gpu_struct
 
     // Identity peer mappings are only defined when
     // peer_copy_mode == UVM_GPU_PEER_COPY_MODE_VIRTUAL
-    uvm_gpu_identity_mapping_t peer_mappings[UVM_MAX_GPUS];
+    uvm_gpu_identity_mapping_t peer_mappings[UVM_ID_MAX_GPUS];
 
     struct
     {
         // Mask of peer_gpus set
+
+
+
+
+
         uvm_processor_mask_t peer_gpu_mask;
 
-        // lazily-populated array of peer GPUs, indexed by peer id
-        uvm_gpu_t *peer_gpus[UVM_MAX_GPUS];
+        // lazily-populated array of peer GPUs, indexed by the peer's GPU index
+        uvm_gpu_t *peer_gpus[UVM_ID_MAX_GPUS];
 
         // Leaf spinlock used to synchronize access to the peer_gpus table so that
         // it can be safely accessed from the access counters bottom half
@@ -716,10 +714,12 @@ struct uvm_gpu_struct
 
         union
         {
-            // Maximum (inclusive) number of single page invalidations before falling back to invalidate all
+            // Maximum (inclusive) number of single page invalidations before
+            // falling back to invalidate all
             NvU32 max_pages;
 
-            // Maximum (inclusive) number of range invalidations before falling back to invalidate all
+            // Maximum (inclusive) number of range invalidations before falling
+            // back to invalidate all
             NvU32 max_ranges;
         };
     } tlb_batch;
@@ -755,8 +755,11 @@ struct uvm_gpu_struct
     // a GPU
     uvm_chunk_sizes_mask_t mmu_kernel_chunk_sizes;
 
+    // RM device handle used in many of the UVM/RM APIs.
+    uvmGpuDeviceHandle rm_device;
+
     // RM address space handle used in many of the UVM/RM APIs
-    // Represents both an RM device and a GPU VA in RM.
+    // Represents a GPU VA space within rm_device.
     uvmGpuAddressSpaceHandle rm_address_space;
 
     // Page tree used for the internal UVM VA space shared with RM
@@ -828,20 +831,9 @@ struct uvm_gpu_struct
     // Access counter buffer info. This is only valid if supports_access_counters is set to true
     uvm_access_counter_buffer_info_t access_counter_buffer_info;
 
-    // Number of GPCs
-    NvU32 gpc_count;
-
-    // Number of TPCs
-    NvU32 tpc_count;
-
-    // Maximum number of TPCs per GPC
-    NvU32 max_tpc_per_gpc;
-
     // Number of uTLBs per GPC. This information is only valid on Pascal+ GPUs.
     NvU32 utlb_per_gpc_count;
 
-    NvU32 max_subcontexts;
-
     // In order to service GPU faults, UVM must be able to obtain the VA
     // space for each reported fault. The fault packet contains the
     // instance_ptr of the channel that was bound when the SMs triggered
@@ -889,9 +881,6 @@ struct uvm_gpu_struct
     // This is set to true if the GPU belongs to an SLI group. Else, set to false.
     bool sli_enabled;
 
-    // This is set to true if the GPU is a simulated/emulated device. Else, set to false.
-    bool is_simulated;
-
     // Global statistics. These fields are per-GPU and most of them are only
     // updated during fault servicing, and can be safely incremented.
     struct
@@ -1025,19 +1014,6 @@ void uvm_gpu_release_locked(uvm_gpu_t *gpu);
 // caller.
 void uvm_gpu_release(uvm_gpu_t *gpu);
 
-// Helper which calls uvm_gpu_retain on each GPU in mask
-void uvm_gpu_retain_mask(const uvm_processor_mask_t *mask);
-
-// Helper which calls uvm_gpu_release_locked on each GPU in mask
-void uvm_gpu_release_mask_locked(const uvm_processor_mask_t *mask);
-
-// Like uvm_gpu_release_mask_locked, but takes and releases the global lock for
-// the caller.
-//
-// LOCKING: this function takes and releases the global lock if the input mask
-//          is not empty
-void uvm_gpu_release_mask(const uvm_processor_mask_t *mask);
-
 static NvU64 uvm_gpu_retained_count(uvm_gpu_t *gpu)
 {
     return atomic64_read(&gpu->retained_count);
@@ -1053,11 +1029,11 @@ NvU32 uvm_gpu_peer_table_index(uvm_gpu_id_t gpu_id1, uvm_gpu_id_t gpu_id2);
 // Either retains an existing PCIe peer entry or creates a new one. In both
 // cases the two GPUs are also each retained.
 // LOCKING: requires the global lock to be held
-NV_STATUS uvm_gpu_retain_pcie_peer_access(uvm_gpu_t *gpu_1, uvm_gpu_t *gpu_2);
+NV_STATUS uvm_gpu_retain_pcie_peer_access(uvm_gpu_t *gpu0, uvm_gpu_t *gpu1);
 
 // Releases a PCIe peer entry and the two GPUs.
 // LOCKING: requires the global lock to be held
-void uvm_gpu_release_pcie_peer_access(uvm_gpu_t *gpu_1, uvm_gpu_t *gpu_2);
+void uvm_gpu_release_pcie_peer_access(uvm_gpu_t *gpu0, uvm_gpu_t *gpu1);
 
 // Get the aperture for local_gpu to use to map memory resident on remote_gpu.
 // They must not be the same gpu.
@@ -1070,9 +1046,9 @@ uvm_processor_id_t uvm_gpu_get_processor_id_by_address(uvm_gpu_t *gpu, uvm_gpu_p
 uvm_gpu_peer_t *uvm_gpu_index_peer_caps(uvm_gpu_id_t gpu_id1, uvm_gpu_id_t gpu_id2);
 
 // Get the P2P capabilities between the given gpus
-static uvm_gpu_peer_t *uvm_gpu_peer_caps(const uvm_gpu_t *gpu_1, const uvm_gpu_t *gpu_2)
+static uvm_gpu_peer_t *uvm_gpu_peer_caps(const uvm_gpu_t *gpu0, const uvm_gpu_t *gpu1)
 {
-    return uvm_gpu_index_peer_caps(gpu_1->id, gpu_2->id);
+    return uvm_gpu_index_peer_caps(gpu0->id, gpu1->id);
 }
 
 static bool uvm_gpus_are_nvswitch_connected(uvm_gpu_t *gpu1, uvm_gpu_t *gpu2)
@@ -1085,29 +1061,31 @@ static bool uvm_gpus_are_nvswitch_connected(uvm_gpu_t *gpu1, uvm_gpu_t *gpu2)
     return false;
 }
 
-static bool uvm_gpus_are_indirect_peers(uvm_gpu_t *gpu1, uvm_gpu_t *gpu2)
+static bool uvm_gpus_are_indirect_peers(uvm_gpu_t *gpu0, uvm_gpu_t *gpu1)
 {
-    uvm_gpu_peer_t *peer_caps = uvm_gpu_peer_caps(gpu1, gpu2);
+    uvm_gpu_peer_t *peer_caps = uvm_gpu_peer_caps(gpu0, gpu1);
 
     if (peer_caps->link_type != UVM_GPU_LINK_INVALID && peer_caps->is_indirect_peer) {
+        UVM_ASSERT(gpu0->numa_info.enabled);
         UVM_ASSERT(gpu1->numa_info.enabled);
-        UVM_ASSERT(gpu2->numa_info.enabled);
         UVM_ASSERT(peer_caps->link_type != UVM_GPU_LINK_PCIE);
-        UVM_ASSERT(!uvm_gpus_are_nvswitch_connected(gpu1, gpu2));
+        UVM_ASSERT(!uvm_gpus_are_nvswitch_connected(gpu0, gpu1));
         return true;
     }
 
     return false;
 }
 
+static uvm_gpu_identity_mapping_t *uvm_gpu_get_peer_mapping(uvm_gpu_t *gpu, uvm_gpu_id_t peer_id)
+{
+    return &gpu->peer_mappings[uvm_id_gpu_index(peer_id)];
+}
+
 // Check for ECC errors
 //
 // Notably this check cannot be performed where it's not safe to call into RM.
 NV_STATUS uvm_gpu_check_ecc_error(uvm_gpu_t *gpu);
 
-// Check for ECC errors for all GPUs in a mask
-NV_STATUS uvm_gpu_check_ecc_error_mask(uvm_processor_mask_t *gpus);
-
 // Check for ECC errors without calling into RM
 //
 // Calling into RM is problematic in many places, this check is always safe to do.
@@ -1138,7 +1116,7 @@ static void uvm_gpu_unmap_cpu_page(uvm_gpu_t *gpu, NvU64 dma_address)
 
 static bool uvm_gpu_is_gk110_plus(uvm_gpu_t *gpu)
 {
-    return gpu->architecture >= NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GK110;
+    return gpu->rm_info.gpuArch >= NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GK110;
 }
 
 // Returns whether the given address is within the GPU's maximum addressable VA
