@@ -386,7 +386,7 @@ static bool tracking_semaphore_check_gpu(uvm_gpu_tracking_semaphore_t *tracking_
     // those cases.
     //
     // But if a pointer is in the table it must match.
-    table_gpu = uvm_gpu_get(gpu->id);
+    table_gpu = uvm_gpu_get(gpu->global_id);
     if (table_gpu)
         UVM_ASSERT(table_gpu == gpu);
 
