@@ -194,6 +194,10 @@ static NV_STATUS test_unlocking_different_lock_order_than_locked(void)
     TEST_CHECK_RET(fake_lock(UVM_LOCK_ORDER_FIRST, UVM_LOCK_FLAGS_MODE_EXCLUSIVE));
     TEST_CHECK_RET(!fake_unlock(UVM_LOCK_ORDER_SECOND, UVM_LOCK_FLAGS_MODE_EXCLUSIVE));
 
+    TEST_CHECK_RET(!__uvm_thread_check_all_unlocked());
+
+    TEST_CHECK_RET(fake_unlock(UVM_LOCK_ORDER_FIRST, UVM_LOCK_FLAGS_MODE_EXCLUSIVE));
+
     TEST_CHECK_RET(__uvm_thread_check_all_unlocked());
 
     return NV_OK;
@@ -438,18 +442,19 @@ static NV_STATUS run_all_lock_tests(void)
 NV_STATUS uvm8_test_lock_sanity(UVM_TEST_LOCK_SANITY_PARAMS *params, struct file *filp)
 {
     NV_STATUS status;
+    uvm_thread_context_wrapper_t thread_context_wrapper_backup;
 
-    // The global PM lock is acquired by the top-level UVM ioctl() entry
-    // point and still held here.  Release it, trusting that no sleep
-    // cycles will be initiated while this test is running, and that no
-    // other UVM system calls are made.  Prior to the introduction of the
-    // global PM lock, this test already assumed that no UVM code paths
-    // that access UVM locks would ever be executed concurrently.
-    uvm_up_read(&g_uvm_global.pm.lock);
+    // The global PM lock is acquired by the top-level UVM ioctl() entry point
+    // and still held here, which confuses the (pre-existing) test logic that
+    // assumes everything is unlocked at the beginning. Clearing the thread
+    // context data resolves the issue, but the original state needs to be saved
+    // and restored before exiting the test to avoid problems in the top-level
+    // code.
+    uvm_thread_context_save(&thread_context_wrapper_backup.context);
 
     status = run_all_lock_tests();
 
-    uvm_down_read(&g_uvm_global.pm.lock);
+    uvm_thread_context_restore(&thread_context_wrapper_backup.context);
 
     return status;
 }
