@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016-2018 NVIDIA Corporation
+    Copyright (c) 2016-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -22,11 +22,10 @@
 *******************************************************************************/
 
 #include "uvm_common.h"
+#include "uvm_ioctl.h"
 #include "uvm_linux.h"
-#include "uvm_linux_ioctl.h"
 #include "uvm8_global.h"
 #include "uvm8_gpu.h"
-#include "uvm8_init.h"
 #include "uvm8_lock.h"
 #include "uvm8_va_space.h"
 #include "uvm8_va_range.h"
@@ -303,7 +302,7 @@ static bool migration_should_do_cpu_preunmap(uvm_va_space_t *va_space,
     if (pass != UVM_MIGRATE_PASS_FIRST || is_single_block)
         return false;
 
-    if (uvm_processor_mask_get_gpu_count(&va_space->has_nvlink[UVM_CPU_ID]) == 0)
+    if (uvm_processor_mask_get_gpu_count(&va_space->has_nvlink[UVM_ID_CPU_VALUE]) == 0)
         return false;
 
     return true;
@@ -346,7 +345,7 @@ static bool va_block_should_do_cpu_preunmap(uvm_va_block_t *va_block,
 
     uvm_mutex_lock(&va_block->lock);
 
-    mapped_pages_cpu = uvm_va_block_map_mask_get(va_block, UVM_CPU_ID);
+    mapped_pages_cpu = uvm_va_block_map_mask_get(va_block, UVM_ID_CPU);
     if (uvm_processor_mask_test(&va_block->resident, dest_id)) {
         const uvm_page_mask_t *resident_pages_dest = uvm_va_block_resident_mask_get(va_block, dest_id);
         uvm_page_mask_t *do_not_unmap_pages = &va_block_context->scratch_page_mask;
@@ -543,11 +542,11 @@ static NV_STATUS uvm_migrate_ranges(uvm_va_space_t *va_space,
             if (!iter.migratable) {
                 // Only return NV_WARN_MORE_PROCESSING_REQUIRED if the pages aren't
                 // already resident at dest_id.
-                if (va_range->preferred_location != dest_id)
+                if (!uvm_id_equal(va_range->preferred_location, dest_id))
                     skipped_migrate = true;
             }
             else if (uvm_processor_mask_test(&va_range->uvm_lite_gpus, dest_id) &&
-                     dest_id != va_range->preferred_location) {
+                     !uvm_id_equal(dest_id, va_range->preferred_location)) {
                 // Don't migrate to a non-faultable GPU that is in UVM-Lite mode,
                 // unless it's the preferred location
                 status = NV_ERR_INVALID_DEVICE;
@@ -603,6 +602,11 @@ static NV_STATUS uvm_migrate(uvm_va_space_t *va_space,
     if (!first_va_range || first_va_range->type != UVM_VA_RANGE_TYPE_MANAGED)
         return NV_ERR_INVALID_ADDRESS;
 
+    // If the GPU has its memory disabled, just skip the migration and let
+    // faults take care of things.
+    if (!uvm_va_space_processor_has_memory(va_space, dest_id))
+        return NV_OK;
+
     va_block_context = uvm_va_block_context_alloc();
     if (!va_block_context)
         return NV_ERR_NO_MEMORY;
@@ -611,8 +615,8 @@ static NV_STATUS uvm_migrate(uvm_va_space_t *va_space,
     // block or UVM_MIGRATE_FLAG_SKIP_CPU_MAP is passed). This helps in the
     // following scenarios:
     //
-    // - Migrations that add CPU mappings, since it is a synchronous operations
-    // and delays the migration of the next VA blocks.
+    // - Migrations that add CPU mappings, since they are synchronous operations
+    // that delay the migration of the next VA blocks.
     // - Concurrent migrations. This is due to our current channel selection
     // logic that doesn't prevent false dependencies between independent
     // operations. For example, removal of mappings for outgoing transfers are
@@ -626,7 +630,7 @@ static NV_STATUS uvm_migrate(uvm_va_space_t *va_space,
     // 2- Go block by block reexecuting the transfer (in case someone moved it
     // since the first pass), and adding the mappings.
     is_single_block = is_migration_single_block(first_va_range, base, length);
-    do_mappings = (dest_id != UVM_CPU_ID) || !(migrate_flags & UVM_MIGRATE_FLAG_SKIP_CPU_MAP);
+    do_mappings = UVM_ID_IS_GPU(dest_id) || !(migrate_flags & UVM_MIGRATE_FLAG_SKIP_CPU_MAP);
     do_two_passes = do_mappings && !is_single_block;
 
     if (do_two_passes) {
@@ -663,10 +667,7 @@ static NV_STATUS uvm_migrate(uvm_va_space_t *va_space,
 
     uvm_va_block_context_free(va_block_context);
 
-    if (status != NV_OK)
-        return status;
-
-    return NV_OK;
+    return status;
 }
 
 static NV_STATUS uvm_push_async_user_sem_release(uvm_gpu_t *release_from_gpu,
@@ -770,7 +771,7 @@ static NV_STATUS uvm_migrate_release_user_sem(const UVM_MIGRATE_PARAMS *params,
             // set up asynchronously, or because of the test-only flag
             // UVM_MIGRATE_FLAG_SKIP_CPU_MAP. However, this means there should be a registered
             // GPU since all CPU work is synchronous.
-            release_from = uvm_processor_mask_find_first_gpu(&va_space->registered_gpus);
+            release_from = uvm_va_space_find_first_gpu(va_space);
             UVM_ASSERT(release_from);
         }
         status = uvm_push_async_user_sem_release(release_from,
@@ -912,7 +913,7 @@ NV_STATUS uvm_api_migrate(UVM_MIGRATE_PARAMS *params, struct file *filp)
             status = uvm_migrate(va_space,
                                  params->base,
                                  params->length,
-                                 (dest_gpu ? dest_gpu->id : UVM_CPU_ID),
+                                 (dest_gpu ? dest_gpu->id : UVM_ID_CPU),
                                  params->flags,
                                  tracker_ptr);
         }
@@ -921,7 +922,7 @@ NV_STATUS uvm_api_migrate(UVM_MIGRATE_PARAMS *params, struct file *filp)
                                           current->mm,
                                           params->base,
                                           params->length,
-                                          (dest_gpu ? dest_gpu->id : UVM_CPU_ID),
+                                          (dest_gpu ? dest_gpu->id : UVM_ID_CPU),
                                           (int)params->cpuNumaNode,
                                           &params->userSpaceStart,
                                           &params->userSpaceLength);
@@ -986,7 +987,7 @@ NV_STATUS uvm_api_migrate_range_group(UVM_MIGRATE_RANGE_GROUP_PARAMS *params, st
     uvm_va_space_down_read(va_space);
 
     if (uvm_uuid_is_cpu(&params->destinationUuid)) {
-        dest_id = UVM_CPU_ID;
+        dest_id = UVM_ID_CPU;
     }
     else {
         gpu = uvm_va_space_get_gpu_by_uuid_with_gpu_va_space(va_space, &params->destinationUuid);
