@@ -1602,10 +1602,10 @@ static NV_STATUS fake_gpu_init(NvU32 host_class, NvU32 ce_class, NvU32 architect
 {
     memset(fake_gpu, 0, sizeof(*fake_gpu));
 
-    fake_gpu->ce_class = ce_class;
-    fake_gpu->host_class = host_class;
-    fake_gpu->architecture = architecture;
-    fake_gpu->fault_buffer_class = fault_buffer_class;
+    fake_gpu->rm_info.ceClass = ce_class;
+    fake_gpu->rm_info.hostClass = host_class;
+    fake_gpu->rm_info.gpuArch = architecture;
+    fake_gpu->rm_info.faultBufferClass = fault_buffer_class;
 
     TEST_CHECK_RET(uvm_hal_init_gpu(fake_gpu) == NV_OK);
 
