@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015 NVIDIA Corporation
+    Copyright (c) 2015-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -21,6 +21,7 @@
 
 *******************************************************************************/
 
+#include "uvm8_api.h"
 #include "uvm8_pushbuffer.h"
 #include "uvm8_channel.h"
 #include "uvm8_lock.h"
@@ -41,7 +42,12 @@ static int nv_procfs_read_pushbuffer_info(struct seq_file *s, void *v)
     return 0;
 }
 
-UVM_DEFINE_SINGLE_PROCFS_FILE(pushbuffer_info);
+static int nv_procfs_read_pushbuffer_info_entry(struct seq_file *s, void *v)
+{
+    UVM_ENTRY_RET(nv_procfs_read_pushbuffer_info(s, v));
+}
+
+UVM_DEFINE_SINGLE_PROCFS_FILE(pushbuffer_info_entry);
 
 static NV_STATUS create_procfs(uvm_pushbuffer_t *pushbuffer)
 {
@@ -51,7 +57,10 @@ static NV_STATUS create_procfs(uvm_pushbuffer_t *pushbuffer)
     if (!uvm_procfs_is_debug_enabled())
         return NV_OK;
 
-    pushbuffer->procfs.info_file = NV_CREATE_PROC_FILE("pushbuffer", gpu->procfs.dir, pushbuffer_info, (void *)pushbuffer);
+    pushbuffer->procfs.info_file = NV_CREATE_PROC_FILE("pushbuffer",
+                                                       gpu->procfs.dir,
+                                                       pushbuffer_info_entry,
+                                                       pushbuffer);
     if (pushbuffer->procfs.info_file == NULL)
         return NV_ERR_OPERATING_SYSTEM;
 
