@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016 NVIDIA Corporation
+    Copyright (c) 2016-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -21,11 +21,11 @@
 
 *******************************************************************************/
 #include "uvm_common.h"
+#include "uvm_ioctl.h"
 #include "uvm8_gpu.h"
 #include "uvm8_hal.h"
 #include "uvm8_tools.h"
 #include "uvm8_va_space.h"
-#include "uvm8_init.h"
 #include "uvm8_api.h"
 #include "uvm8_hal_types.h"
 #include "uvm8_va_block.h"
@@ -588,6 +588,11 @@ static int uvm_tools_open(struct inode *inode, struct file *filp)
     return -nv_status_to_errno(uvm_global_get_status());
 }
 
+static int uvm_tools_open_entry(struct inode *inode, struct file *filp)
+{
+    UVM_ENTRY_RET(uvm_tools_open(inode, filp));
+}
+
 static int uvm_tools_release(struct inode *inode, struct file *filp)
 {
     uvm_tools_event_tracker_t *event_tracker = tools_event_tracker(filp);
@@ -598,6 +603,11 @@ static int uvm_tools_release(struct inode *inode, struct file *filp)
     return -nv_status_to_errno(uvm_global_get_status());
 }
 
+static int uvm_tools_release_entry(struct inode *inode, struct file *filp)
+{
+    UVM_ENTRY_RET(uvm_tools_release(inode, filp));
+}
+
 static long uvm_tools_unlocked_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 {
     switch (cmd) {
@@ -614,7 +624,12 @@ static long uvm_tools_unlocked_ioctl(struct file *filp, unsigned int cmd, unsign
     return -EINVAL;
 }
 
-static unsigned int uvm_tools_poll(struct file *filp, poll_table *wait)
+static long uvm_tools_unlocked_ioctl_entry(struct file *filp, unsigned int cmd, unsigned long arg)
+{
+    UVM_ENTRY_RET(uvm_tools_unlocked_ioctl(filp, cmd, arg));
+}
+
+static unsigned uvm_tools_poll(struct file *filp, poll_table *wait)
 {
     int flags = 0;
     uvm_tools_queue_snapshot_t sn;
@@ -644,6 +659,11 @@ static unsigned int uvm_tools_poll(struct file *filp, poll_table *wait)
     return flags;
 }
 
+static unsigned uvm_tools_poll_entry(struct file *filp, poll_table *wait)
+{
+    UVM_ENTRY_RET(uvm_tools_poll(filp, wait));
+}
+
 static UvmEventFaultType g_hal_to_tools_fault_type_table[UVM_FAULT_TYPE_COUNT] = {
     [UVM_FAULT_TYPE_INVALID_PDE]          = UvmFaultTypeInvalidPde,
     [UVM_FAULT_TYPE_INVALID_PTE]          = UvmFaultTypeInvalidPte,
@@ -700,7 +720,7 @@ static void record_gpu_fault_instance(uvm_gpu_t *gpu,
     memset(&entry, 0, sizeof(entry));
 
     info->eventType     = UvmEventTypeGpuFault;
-    info->gpuIndex      = gpu->id;
+    info->gpuIndex      = uvm_id_value(gpu->id);
     info->faultType     = g_hal_to_tools_fault_type_table[fault_entry->fault_type];
     info->accessType    = g_hal_to_tools_fault_access_type_table[fault_entry->fault_access_type];
     info->clientType    = g_hal_to_tools_fault_client_type_table[fault_entry->fault_source.client_type];
@@ -731,7 +751,7 @@ static void uvm_tools_record_fault(uvm_perf_event_t event_id, uvm_perf_event_dat
     uvm_down_read(&va_space->tools.lock);
     UVM_ASSERT(tools_is_fault_callback_needed(va_space));
 
-    if (event_data->fault.proc_id == UVM_CPU_ID) {
+    if (UVM_ID_IS_CPU(event_data->fault.proc_id)) {
         if (tools_is_event_enabled(va_space, UvmEventTypeCpuFault)) {
             UvmEventEntry entry;
             UvmEventCpuFaultInfo *info = &entry.eventData.cpuFault;
@@ -764,14 +784,14 @@ static void uvm_tools_record_fault(uvm_perf_event_t event_id, uvm_perf_event_dat
             uvm_tools_inc_counter(va_space, UvmCounterNameCpuPageFaultCount, 1, &NV_PROCESSOR_UUID_CPU_DEFAULT);
 
             preferred_location = event_data->fault.block->va_range->preferred_location;
-            if (preferred_location != UVM_MAX_PROCESSORS && preferred_location != UVM_CPU_ID) {
-                uvm_gpu_t *gpu = uvm_gpu_get(preferred_location);
+            if (UVM_ID_IS_GPU(preferred_location)) {
+                uvm_gpu_t *gpu = uvm_va_space_get_gpu(va_space, preferred_location);
                 uvm_tools_inc_counter(va_space, UvmCounterNameCpuPageFaultCount, 1, &gpu->uuid);
             }
         }
     }
     else {
-        uvm_gpu_t *gpu = uvm_gpu_get(event_data->fault.proc_id);
+        uvm_gpu_t *gpu = uvm_va_space_get_gpu(va_space, event_data->fault.proc_id);
         UVM_ASSERT(gpu);
 
         if (tools_is_event_enabled(va_space, UvmEventTypeGpuFault)) {
@@ -821,23 +841,23 @@ static void record_migration_events(void *args)
 
     // Initialize fields that are constant throughout the whole block
     memset(&entry, 0, sizeof(entry));
-    info->eventType = UvmEventTypeMigration;
-    info->srcIndex = block_mig->src;
-    info->dstIndex = block_mig->dst;
+    info->eventType      = UvmEventTypeMigration;
+    info->srcIndex       = uvm_id_value(block_mig->src);
+    info->dstIndex       = uvm_id_value(block_mig->dst);
     info->beginTimeStamp = block_mig->start_timestamp_cpu;
-    info->endTimeStamp = block_mig->end_timestamp_cpu;
-    info->rangeGroupId = block_mig->range_group_id;
+    info->endTimeStamp   = block_mig->end_timestamp_cpu;
+    info->rangeGroupId   = block_mig->range_group_id;
 
     uvm_down_read(&va_space->tools.lock);
     list_for_each_entry_safe(mig, next, &block_mig->events, events_node) {
         UVM_ASSERT(mig->bytes > 0);
         list_del(&mig->events_node);
 
-        info->address = mig->address;
-        info->migratedBytes = mig->bytes;
+        info->address           = mig->address;
+        info->migratedBytes     = mig->bytes;
         info->beginTimeStampGpu = gpu_timestamp;
-        info->endTimeStampGpu = mig->end_timestamp_gpu;
-        info->migrationCause = mig->cause;
+        info->endTimeStampGpu   = mig->end_timestamp_gpu;
+        info->migrationCause    = mig->cause;
         gpu_timestamp = mig->end_timestamp_gpu;
         kmem_cache_free(g_tools_migration_data_cache, mig);
 
@@ -849,6 +869,11 @@ static void record_migration_events(void *args)
     kmem_cache_free(g_tools_block_migration_data_cache, block_mig);
 }
 
+static void record_migration_events_entry(void *args)
+{
+    UVM_ENTRY_VOID(record_migration_events(args));
+}
+
 static void on_block_migration_complete(void *ptr)
 {
     migration_data_t *mig;
@@ -859,7 +884,7 @@ static void on_block_migration_complete(void *ptr)
     list_for_each_entry(mig, &block_mig->events, events_node)
         mig->end_timestamp_gpu = *mig->end_timestamp_gpu_addr;
 
-    nv_kthread_q_item_init(&block_mig->queue_item, record_migration_events, block_mig);
+    nv_kthread_q_item_init(&block_mig->queue_item, record_migration_events_entry, block_mig);
 
     // The UVM driver may notice that work in a channel is complete in a variety of situations
     // and the va_space lock is not always held in all of them, nor can it always be taken safely on them.
@@ -881,11 +906,11 @@ static void record_replay_event_helper(uvm_gpu_id_t gpu_id,
     UvmEventEntry entry;
 
     memset(&entry, 0, sizeof(entry));
-    entry.eventData.gpuFaultReplay.eventType = UvmEventTypeGpuFaultReplay;
-    entry.eventData.gpuFaultReplay.gpuIndex = gpu_id;
-    entry.eventData.gpuFaultReplay.batchId = batch_id;
-    entry.eventData.gpuFaultReplay.clientType = g_hal_to_tools_fault_client_type_table[client_type];
-    entry.eventData.gpuFaultReplay.timeStamp = timestamp;
+    entry.eventData.gpuFaultReplay.eventType    = UvmEventTypeGpuFaultReplay;
+    entry.eventData.gpuFaultReplay.gpuIndex     = uvm_id_value(gpu_id);
+    entry.eventData.gpuFaultReplay.batchId      = batch_id;
+    entry.eventData.gpuFaultReplay.clientType   = g_hal_to_tools_fault_client_type_table[client_type];
+    entry.eventData.gpuFaultReplay.timeStamp    = timestamp;
     entry.eventData.gpuFaultReplay.timeStampGpu = timestamp_gpu;
 
     uvm_tools_broadcast_event(&entry);
@@ -904,12 +929,17 @@ static void record_replay_events(void *args)
     kmem_cache_free(g_tools_replay_data_cache, replay);
 }
 
+static void record_replay_events_entry(void *args)
+{
+    UVM_ENTRY_VOID(record_replay_events(args));
+}
+
 static void on_replay_complete(void *ptr)
 {
     replay_data_t *replay = (replay_data_t *)ptr;
     replay->timestamp_gpu = *replay->timestamp_gpu_addr;
 
-    nv_kthread_q_item_init(&replay->queue_item, record_replay_events, ptr);
+    nv_kthread_q_item_init(&replay->queue_item, record_replay_events_entry, ptr);
 
     uvm_spin_lock(&g_tools_channel_list_lock);
     remove_pending_event_for_channel(replay->channel);
@@ -967,12 +997,14 @@ static void uvm_tools_record_migration(uvm_perf_event_t event_id, uvm_perf_event
     }
 
     // Increment counters
-    if (event_data->migration.src == UVM_CPU_ID && tools_is_counter_enabled(va_space, UvmCounterNameBytesXferHtD)) {
-        uvm_gpu_t *gpu = uvm_gpu_get(event_data->migration.dst);
+    if (UVM_ID_IS_CPU(event_data->migration.src) &&
+        tools_is_counter_enabled(va_space, UvmCounterNameBytesXferHtD)) {
+        uvm_gpu_t *gpu = uvm_va_space_get_gpu(va_space, event_data->migration.dst);
         uvm_tools_inc_counter(va_space, UvmCounterNameBytesXferHtD, event_data->migration.bytes, &gpu->uuid);
     }
-    if (event_data->migration.dst == UVM_CPU_ID && tools_is_counter_enabled(va_space, UvmCounterNameBytesXferDtH)) {
-        uvm_gpu_t *gpu = uvm_gpu_get(event_data->migration.src);
+    if (UVM_ID_IS_CPU(event_data->migration.dst) &&
+        tools_is_counter_enabled(va_space, UvmCounterNameBytesXferDtH)) {
+        uvm_gpu_t *gpu = uvm_va_space_get_gpu(va_space, event_data->migration.src);
         uvm_tools_inc_counter(va_space, UvmCounterNameBytesXferDtH, event_data->migration.bytes, &gpu->uuid);
     }
 
@@ -1002,11 +1034,11 @@ void uvm_tools_broadcast_replay(uvm_gpu_t *gpu,
     UVM_ASSERT(push_info->on_complete == NULL && push_info->on_complete_data == NULL);
 
     replay->timestamp_gpu_addr = uvm_push_timestamp(push);
-    replay->gpu_id = gpu->id;
-    replay->batch_id = batch_id;
-    replay->client_type = client_type;
-    replay->timestamp = NV_GETTIME();
-    replay->channel = push->channel;
+    replay->gpu_id             = gpu->id;
+    replay->batch_id           = batch_id;
+    replay->client_type        = client_type;
+    replay->timestamp          = NV_GETTIME();
+    replay->channel            = push->channel;
 
     push_info->on_complete_data = replay;
     push_info->on_complete = on_replay_complete;
@@ -1048,12 +1080,12 @@ void uvm_tools_broadcast_access_counter(uvm_gpu_t *gpu,
         return;
 
     if (!buffer_entry->address.is_virtual)
-        UVM_ASSERT(buffer_entry->physical_info.resident_id != UVM_MAX_PROCESSORS);
+        UVM_ASSERT(UVM_ID_IS_VALID(buffer_entry->physical_info.resident_id));
 
     memset(&entry, 0, sizeof(entry));
 
     info->eventType           = UvmEventTypeTestAccessCounter;
-    info->srcIndex            = gpu->id;
+    info->srcIndex            = uvm_id_value(gpu->id);
     info->address             = buffer_entry->address.address;
     info->isVirtual           = buffer_entry->address.is_virtual? 1: 0;
     if (buffer_entry->address.is_virtual) {
@@ -1174,7 +1206,7 @@ void uvm_tools_record_read_duplicate(uvm_va_block_t *va_block,
 
             uvm_va_block_page_resident_processors(va_block, page_index, &resident_processors);
             for_each_id_in_mask(id, &resident_processors)
-                info_read_duplicate->processors |= (1 << id);
+                info_read_duplicate->processors |= (1 << uvm_id_value(id));
 
             uvm_tools_record_event(va_space, &entry);
         }
@@ -1200,7 +1232,7 @@ void uvm_tools_record_read_duplicate_invalidate(uvm_va_block_t *va_block,
         memset(&entry, 0, sizeof(entry));
 
         info->eventType     = UvmEventTypeReadDuplicateInvalidate;
-        info->residentIndex = dst;
+        info->residentIndex = uvm_id_value(dst);
         info->size          = PAGE_SIZE;
         info->timeStamp     = NV_GETTIME();
 
@@ -1269,13 +1301,13 @@ void uvm_tools_record_cpu_fatal_fault(uvm_va_space_t *va_space,
         UvmEventFatalFaultInfo *info = &entry.eventData.fatalFault;
         memset(&entry, 0, sizeof(entry));
 
-        info->eventType = UvmEventTypeFatalFault;
-        info->processorIndex = UVM_CPU_ID;
-        info->timeStamp = NV_GETTIME();
-        info->address = address;
-        info->accessType = is_write? UvmEventMemoryAccessTypeWrite: UvmEventMemoryAccessTypeRead;
+        info->eventType      = UvmEventTypeFatalFault;
+        info->processorIndex = UVM_ID_CPU_VALUE;
+        info->timeStamp      = NV_GETTIME();
+        info->address        = address;
+        info->accessType     = is_write? UvmEventMemoryAccessTypeWrite: UvmEventMemoryAccessTypeRead;
         // info->faultType is not valid for cpu faults
-        info->reason = reason;
+        info->reason         = reason;
 
         uvm_tools_record_event(va_space, &entry);
     }
@@ -1298,13 +1330,13 @@ void uvm_tools_record_gpu_fatal_fault(uvm_gpu_id_t gpu_id,
         UvmEventFatalFaultInfo *info = &entry.eventData.fatalFault;
         memset(&entry, 0, sizeof(entry));
 
-        info->eventType = UvmEventTypeFatalFault;
-        info->processorIndex = gpu_id;
-        info->timeStamp = NV_GETTIME();
-        info->address = buffer_entry->fault_address;
-        info->accessType = g_hal_to_tools_fault_access_type_table[buffer_entry->fault_access_type];
-        info->faultType = g_hal_to_tools_fault_type_table[buffer_entry->fault_type];
-        info->reason = reason;
+        info->eventType      = UvmEventTypeFatalFault;
+        info->processorIndex = uvm_id_value(gpu_id);
+        info->timeStamp      = NV_GETTIME();
+        info->address        = buffer_entry->fault_address;
+        info->accessType     = g_hal_to_tools_fault_access_type_table[buffer_entry->fault_access_type];
+        info->faultType      = g_hal_to_tools_fault_type_table[buffer_entry->fault_type];
+        info->reason         = reason;
 
         uvm_tools_record_event(va_space, &entry);
     }
@@ -1335,7 +1367,7 @@ void uvm_tools_record_thrashing(uvm_va_space_t *va_space,
         info->address   = address;
         info->size      = region_size;
         info->timeStamp = NV_GETTIME();
-        bitmap_copy((long unsigned *)&info->processors, processors->bitmap, UVM_MAX_PROCESSORS);
+        bitmap_copy((long unsigned *)&info->processors, processors->bitmap, UVM_ID_MAX_PROCESSORS);
 
         uvm_tools_record_event(va_space, &entry);
     }
@@ -1346,7 +1378,7 @@ void uvm_tools_record_throttling_start(uvm_va_space_t *va_space, NvU64 address,
 {
     UVM_ASSERT(address);
     UVM_ASSERT(PAGE_ALIGNED(address));
-    UVM_ASSERT(processor < UVM_MAX_PROCESSORS);
+    UVM_ASSERT(UVM_ID_IS_VALID(processor));
 
     uvm_assert_rwsem_locked(&va_space->lock);
 
@@ -1360,7 +1392,7 @@ void uvm_tools_record_throttling_start(uvm_va_space_t *va_space, NvU64 address,
         memset(&entry, 0, sizeof(entry));
 
         info->eventType      = UvmEventTypeThrottlingStart;
-        info->processorIndex = processor;
+        info->processorIndex = uvm_id_value(processor);
         info->address        = address;
         info->timeStamp      = NV_GETTIME();
 
@@ -1373,7 +1405,7 @@ void uvm_tools_record_throttling_end(uvm_va_space_t *va_space, NvU64 address, uv
 {
     UVM_ASSERT(address);
     UVM_ASSERT(PAGE_ALIGNED(address));
-    UVM_ASSERT(processor < UVM_MAX_PROCESSORS);
+    UVM_ASSERT(UVM_ID_IS_VALID(processor));
 
     uvm_assert_rwsem_locked(&va_space->lock);
 
@@ -1387,7 +1419,7 @@ void uvm_tools_record_throttling_end(uvm_va_space_t *va_space, NvU64 address, uv
         memset(&entry, 0, sizeof(entry));
 
         info->eventType      = UvmEventTypeThrottlingEnd;
-        info->processorIndex = processor;
+        info->processorIndex = uvm_id_value(processor);
         info->address        = address;
         info->timeStamp      = NV_GETTIME();
 
@@ -1405,18 +1437,18 @@ static void record_map_remote_events(void *args)
 
     memset(&entry, 0, sizeof(entry));
 
-    entry.eventData.mapRemote.eventType = UvmEventTypeMapRemote;
-    entry.eventData.mapRemote.srcIndex = block_map_remote->src;
-    entry.eventData.mapRemote.dstIndex = block_map_remote->dst;
+    entry.eventData.mapRemote.eventType      = UvmEventTypeMapRemote;
+    entry.eventData.mapRemote.srcIndex       = uvm_id_value(block_map_remote->src);
+    entry.eventData.mapRemote.dstIndex       = uvm_id_value(block_map_remote->dst);
     entry.eventData.mapRemote.mapRemoteCause = block_map_remote->cause;
-    entry.eventData.mapRemote.timeStamp = block_map_remote->timestamp;
+    entry.eventData.mapRemote.timeStamp      = block_map_remote->timestamp;
 
     uvm_down_read(&va_space->tools.lock);
     list_for_each_entry_safe(map_remote, next, &block_map_remote->events, events_node) {
         list_del(&map_remote->events_node);
 
-        entry.eventData.mapRemote.address = map_remote->address;
-        entry.eventData.mapRemote.size = map_remote->size;
+        entry.eventData.mapRemote.address      = map_remote->address;
+        entry.eventData.mapRemote.size         = map_remote->size;
         entry.eventData.mapRemote.timeStampGpu = map_remote->timestamp_gpu;
         kmem_cache_free(g_tools_map_remote_data_cache, map_remote);
 
@@ -1428,17 +1460,22 @@ static void record_map_remote_events(void *args)
     kmem_cache_free(g_tools_block_map_remote_data_cache, block_map_remote);
 }
 
+static void record_map_remote_events_entry(void *args)
+{
+    UVM_ENTRY_VOID(record_map_remote_events(args));
+}
+
 static void on_map_remote_complete(void *ptr)
 {
     block_map_remote_data_t *block_map_remote = (block_map_remote_data_t *)ptr;
     map_remote_data_t *map_remote;
 
     // Only GPU mappings use the deferred mechanism
-    UVM_ASSERT(block_map_remote->src != UVM_CPU_ID);
+    UVM_ASSERT(UVM_ID_IS_GPU(block_map_remote->src));
     list_for_each_entry(map_remote, &block_map_remote->events, events_node)
         map_remote->timestamp_gpu = *map_remote->timestamp_gpu_addr;
 
-    nv_kthread_q_item_init(&block_map_remote->queue_item, record_map_remote_events, ptr);
+    nv_kthread_q_item_init(&block_map_remote->queue_item, record_map_remote_events_entry, ptr);
 
     uvm_spin_lock(&g_tools_channel_list_lock);
     remove_pending_event_for_channel(block_map_remote->channel);
@@ -1456,8 +1493,8 @@ void uvm_tools_record_map_remote(uvm_va_block_t *va_block,
 {
     uvm_va_space_t *va_space = va_block->va_range->va_space;
 
-    UVM_ASSERT(processor < UVM_MAX_PROCESSORS);
-    UVM_ASSERT(residency < UVM_MAX_PROCESSORS);
+    UVM_ASSERT(UVM_ID_IS_VALID(processor));
+    UVM_ASSERT(UVM_ID_IS_VALID(residency));
     UVM_ASSERT(cause != UvmEventMapRemoteCauseInvalid);
 
     uvm_assert_rwsem_locked(&va_space->lock);
@@ -1469,19 +1506,20 @@ void uvm_tools_record_map_remote(uvm_va_block_t *va_block,
     if (!tools_is_event_enabled(va_space, UvmEventTypeMapRemote))
         goto done;
 
-    if (processor == UVM_CPU_ID) {
+    if (UVM_ID_IS_CPU(processor)) {
         UvmEventEntry entry;
         memset(&entry, 0, sizeof(entry));
 
-        entry.eventData.mapRemote.eventType = UvmEventTypeMapRemote;
-        entry.eventData.mapRemote.srcIndex = processor;
-        entry.eventData.mapRemote.dstIndex = residency;
+        entry.eventData.mapRemote.eventType      = UvmEventTypeMapRemote;
+        entry.eventData.mapRemote.srcIndex       = uvm_id_value(processor);
+        entry.eventData.mapRemote.dstIndex       = uvm_id_value(residency);
         entry.eventData.mapRemote.mapRemoteCause = cause;
+        entry.eventData.mapRemote.timeStamp      = NV_GETTIME();
+        entry.eventData.mapRemote.address        = address;
+        entry.eventData.mapRemote.size           = region_size;
+        entry.eventData.mapRemote.timeStampGpu   = 0;
+
         UVM_ASSERT(entry.eventData.mapRemote.mapRemoteCause != UvmEventMapRemoteCauseInvalid);
-        entry.eventData.mapRemote.timeStamp = NV_GETTIME();
-        entry.eventData.mapRemote.address = address;
-        entry.eventData.mapRemote.size = region_size;
-        entry.eventData.mapRemote.timeStampGpu = 0;
 
         uvm_tools_record_event(va_space, &entry);
     }
@@ -1964,8 +2002,13 @@ static NV_STATUS tools_access_process_memory(uvm_va_space_t *va_space,
 
         // For simplicity, check for ECC errors on all GPUs registered in the VA
         // space as tools read/write is not on a perf critical path.
-        if (status == NV_OK)
-            status = uvm_gpu_check_ecc_error_mask(&va_space->registered_gpus);
+        if (status == NV_OK) {
+            uvm_global_processor_mask_t global_gpus;
+
+            uvm_va_space_global_gpus(va_space, &global_gpus);
+
+            status = uvm_global_mask_check_ecc_error(&global_gpus);
+        }
 
         uvm_va_space_up_read_rm(va_space);
         if (status != NV_OK)
@@ -2056,18 +2099,18 @@ NV_STATUS uvm_api_tools_get_processor_uuid_table(UVM_TOOLS_GET_PROCESSOR_UUID_TA
     uvm_gpu_t *gpu;
     uvm_va_space_t *va_space = uvm_va_space_get(filp);
 
-    uuids = uvm_kvmalloc_zero(sizeof(NvProcessorUuid) * UVM_MAX_PROCESSORS);
+    uuids = uvm_kvmalloc_zero(sizeof(NvProcessorUuid) * UVM_ID_MAX_PROCESSORS);
     if (uuids == NULL)
         return NV_ERR_NO_MEMORY;
 
-    uvm_processor_uuid_copy(&uuids[UVM_CPU_ID], &NV_PROCESSOR_UUID_CPU_DEFAULT);
+    uvm_processor_uuid_copy(&uuids[UVM_ID_CPU_VALUE], &NV_PROCESSOR_UUID_CPU_DEFAULT);
     params->count = 1;
 
     uvm_va_space_down_read(va_space);
     for_each_va_space_gpu(gpu, va_space) {
-       uvm_processor_uuid_copy(&uuids[gpu->id], &gpu->uuid);
-       if (gpu->id + 1 > params->count)
-           params->count = gpu->id + 1;
+       uvm_processor_uuid_copy(&uuids[uvm_id_value(gpu->id)], &gpu->uuid);
+       if (uvm_id_value(gpu->id) + 1 > params->count)
+           params->count = uvm_id_value(gpu->id) + 1;
     }
     uvm_va_space_up_read(va_space);
 
@@ -2095,13 +2138,13 @@ NV_STATUS uvm_api_tools_flush_events(UVM_TOOLS_FLUSH_EVENTS_PARAMS *params, stru
 
 static const struct file_operations uvm_tools_fops =
 {
-    .open            = uvm_tools_open,
-    .release         = uvm_tools_release,
-    .unlocked_ioctl  = uvm_tools_unlocked_ioctl,
-#if NVCPU_IS_X86_64 && defined(NV_FILE_OPERATIONS_HAS_COMPAT_IOCTL)
-    .compat_ioctl    = uvm_tools_unlocked_ioctl,
+    .open            = uvm_tools_open_entry,
+    .release         = uvm_tools_release_entry,
+    .unlocked_ioctl  = uvm_tools_unlocked_ioctl_entry,
+#if NVCPU_IS_X86_64
+    .compat_ioctl    = uvm_tools_unlocked_ioctl_entry,
 #endif
-    .poll            = uvm_tools_poll,
+    .poll            = uvm_tools_poll_entry,
     .owner           = THIS_MODULE,
 };
 
