@@ -47,6 +47,7 @@
 #include <linux/prio_tree.h>
 #endif
 
+#include <linux/jhash.h>
 #include <linux/rwsem.h>
 #include <linux/rbtree.h>
 #include <asm/current.h>
@@ -74,6 +75,31 @@
 #include <linux/sched/task_stack.h>
 #endif
 
+#include <linux/cpumask.h>
+#include <linux/topology.h>
+
+#include "nv-kthread-q.h"
+
+#if NV_KTHREAD_Q_SUPPORTS_AFFINITY() == 1 && defined(NV_CPUMASK_OF_NODE_PRESENT)
+#define UVM_THREAD_AFFINITY_SUPPORTED() 1
+#else
+#define UVM_THREAD_AFFINITY_SUPPORTED() 0
+#endif
+
+// The ARM arch lacks support for cpumask_of_node() until kernel 4.7. It was
+// added via commit1a2db300348b ("arm64, numa: Add NUMA support for arm64
+// platforms.") Callers should either check UVM_THREAD_AFFINITY_SUPPORTED()
+// prior to calling this function of be prepared to deal with a NULL CPU
+// mask.
+static inline const struct cpumask *uvm_cpumask_of_node(int node)
+{
+#ifdef NV_CPUMASK_OF_NODE_PRESENT
+    return cpumask_of_node(node);
+#else
+    return NULL;
+#endif
+}
+
 // TODO: Bug 1772628: remove the "defined(NV_BUILD_SUPPORTS_HMM)" part,
 // once the HMM (Heterogeneous Memory Management Linux kernel feature) patch
 // gets submitted to the Linux kernel.
@@ -183,18 +209,6 @@
     void address_space_init_once(struct address_space *mapping);
 #endif
 
-#if !defined(NV_FATAL_SIGNAL_PENDING_PRESENT)
-    static inline int __fatal_signal_pending(struct task_struct *p)
-    {
-        return unlikely(sigismember(&p->pending.signal, SIGKILL));
-    }
-
-    static inline int fatal_signal_pending(struct task_struct *p)
-    {
-        return signal_pending(p) && __fatal_signal_pending(p);
-    }
-#endif
-
 // Develop builds define DEBUG but enable optimization
 #if defined(DEBUG) && !defined(NVIDIA_UVM_DEVELOP)
   // Wrappers for functions not building correctly without optimizations on,
@@ -236,36 +250,10 @@
   #define nv_copy_from_user         copy_from_user
 #endif
 
-#if defined(NV_ATOMIC64_PRESENT)
-typedef atomic64_t NV_ATOMIC64;
-#define NV_ATOMIC64_INC(data)           atomic64_inc(&(data))
-#define NV_ATOMIC64_SET(data,val)       atomic64_set(&(data), (val))
-#define NV_ATOMIC64_READ(data)          atomic64_read(&(data))
-#else
-#warning "atomic64_t unavailable, demoting to atomic_t!"
-typedef atomic_t NV_ATOMIC64;
-#define NV_ATOMIC64_INC(data)           atomic_inc(&(data))
-#define NV_ATOMIC64_SET(data,val)       atomic_set(&(data), (val))
-#define NV_ATOMIC64_READ(data)          atomic_read(&(data))
-#endif
-
 #ifndef NV_ALIGN_DOWN
 #define NV_ALIGN_DOWN(v,g) ((v) & ~((g) - 1))
 #endif
 
-//
-// This provides a value that can be used where vmf->flags would normally
-// be used, but on older kernels that do not have a vmf, nor FAULT_FLAG_*
-// definitions:
-//
-#define FAULT_FLAG_FROM_OLD_KERNEL     0x80000000
-
-#if defined(NV_FAULT_FLAG_PRESENT)
-#define NV_FAULT_FLAG_WRITE            FAULT_FLAG_WRITE
-#else
-#define NV_FAULT_FLAG_WRITE            0x01
-#endif
-
 #if !defined(NV_KBASENAME_PRESENT)
 static inline const char *kbasename(const char *str)
 {
@@ -500,16 +488,4 @@ static void __sched usleep_range(unsigned long min, unsigned long max)
 }
 #endif
 
-static bool uvm_memory_node_is_valid(int node_id)
-{
-    if (node_id < 0 || node_id >= MAX_NUMNODES)
-        return false;
-#if defined(NV_NODE_STATES_N_MEMORY_PRESENT)
-    return node_state(node_id, N_MEMORY);
-#else
-    return node_state(node_id, N_HIGH_MEMORY);
-#endif
-}
-
 #endif // _UVM_LINUX_H
-
