@@ -27,9 +27,9 @@ static NV_STATUS nv_dma_map_contig(
     NvU64 *va
 )
 {
-    *va = pci_map_page(dma_map->pci_dev, dma_map->pages[0], 0,
-            dma_map->page_count * PAGE_SIZE, PCI_DMA_BIDIRECTIONAL);
-    if (pci_dma_mapping_error(dma_map->pci_dev, *va))
+    *va = dma_map_page(dma_map->dev, dma_map->pages[0], 0,
+            dma_map->page_count * PAGE_SIZE, DMA_BIDIRECTIONAL);
+    if (dma_mapping_error(dma_map->dev, *va))
     {
         return NV_ERR_OPERATING_SYSTEM;
     }
@@ -54,8 +54,8 @@ static NV_STATUS nv_dma_map_contig(
 
 static void nv_dma_unmap_contig(nv_dma_map_t *dma_map)
 {
-    pci_unmap_page(dma_map->pci_dev, dma_map->mapping.contig.dma_addr,
-            dma_map->page_count * PAGE_SIZE, PCI_DMA_BIDIRECTIONAL);
+    dma_unmap_page(dma_map->dev, dma_map->mapping.contig.dma_addr,
+            dma_map->page_count * PAGE_SIZE, DMA_BIDIRECTIONAL);
 }
 
 static void nv_fill_scatterlist
@@ -163,10 +163,10 @@ NV_STATUS nv_map_dma_map_scatterlist(nv_dma_map_t *dma_map)
 
     NV_FOR_EACH_DMA_SUBMAP(dma_map, submap, i)
     {
-        submap->sg_map_count = pci_map_sg(dma_map->pci_dev,
+        submap->sg_map_count = dma_map_sg(dma_map->dev,
                 submap->sgt.sgl,
                 submap->sgt.orig_nents,
-                PCI_DMA_BIDIRECTIONAL);
+                DMA_BIDIRECTIONAL);
         if (submap->sg_map_count == 0)
         {
             status = NV_ERR_OPERATING_SYSTEM;
@@ -193,10 +193,9 @@ void nv_unmap_dma_map_scatterlist(nv_dma_map_t *dma_map)
         {
             break;
         }
-
-        pci_unmap_sg(dma_map->pci_dev, submap->sgt.sgl,
+        dma_unmap_sg(dma_map->dev, submap->sgt.sgl,
                 submap->sgt.orig_nents,
-                PCI_DMA_BIDIRECTIONAL);
+                DMA_BIDIRECTIONAL);
     }
 }
 
@@ -407,7 +406,7 @@ NV_STATUS NV_API_CALL nv_dma_map_pages(
         return status;
     }
 
-    dma_map->pci_dev = nvl->pci_dev;
+    dma_map->dev = nvl->dev;
     dma_map->pages = *priv;
     dma_map->page_count = page_count;
     dma_map->contiguous = contig;
